#pragma rtGlobals=1		// Use modern global access method.

// version 2015
// to do: image 2D autocorrelation function 
// drift correction waves
// pwave:		"DC"+datafolder+"_P"
// qwave:		"DC"+datafolder+"_Q"
// table:		"DC"+datafolder

Menu "Drift Correction"
	"Reset"
	"CursorAB_Drift_One_Layer"
	"CursorAB_Drift_Stack"
	"Preview_Correction"
	"Do_DriftCorrection"
	 "Coadd_Layers"
	 " coAddAroundALayer"
End

Proc Reset()
	DCR_reset()
End

Proc CursorAB_Drift_One_Layer()
	DCR_csrAB_layer()
End

Proc CursorAB_Drift_Stack()
	DCR_csrAB_Drift()
	DCR_doDriftCorrection()
End
 
Proc  Preview_Correction()
 	DCR_preview()
 End
 
Proc Do_DriftCorrection()
	DCR_doDriftCorrection()
End

Proc Coadd_layers()
	DCR_Coadd_Layers()
End

Proc coAddAroundALayer()
	DCR_coAddAroundALayer()
End

Function DCR_reset()
	// data folder and stack
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	String stackStr=SIS_getImageStackName()
	wave wstack=$stackStr
	variable nr=dimSize($stackStr,2)
	// create drift waves 
	String pStr="DC"+stackStr+"_P"
	String qStr="DC"+stackStr+"_Q"
	if ((exists(pStr)==0)&&(exists(qStr)==0))
		make/n=(nr)/o $pStr,$qStr 
	else
		redimension/n=(nr) $pStr,$qStr
	endif
	wave pw=$pStr
	wave qw=$qStr
	String tableStr="DC"+stackStr
	pw=0
	qw=0
	SIS_tableToFront(tableStr,pStr)
	SIS_tableToFront(tableStr,qStr)
End
 
Function DCR_csrAB_Drift()
	// data folder and stack
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	String stackStr=SIS_getImageStackName()
	wave wstack=$stackStr
	variable nr=dimSize($stackStr,2)
	// pause for cursors
	String winStr=SIS_getWinName()
	ShowInfo
	SIS_PauseForCursors(winStr,"csrA (B) at start (stop)")
	// create drift waves 
	String pStr="DC"+stackStr+"_P"
	String qStr="DC"+stackStr+"_Q"
	if ((exists(pStr)==0)&&(exists(qStr)==0))
		make/n=(nr)/o $pStr,$qStr 
	else
		redimension/n=(nr) $pStr,$qStr
	endif
	wave pw=$pStr
	wave qw=$qStr
	String tableStr="DC"+stackStr
	// check that cursor A and B are on the image
	if ((stringmatch(CsrInfo(A),"")==0) && (stringmatch(CsrInfo(B),"")==0))
		// cursor positions
		variable q0=qcsr(A)
		variable q1=qcsr(B)
		variable p0=pcsr(A)
		variable p1=pcsr(B)
		// input 
		variable firstlayer=0
		prompt firstlayer, "Start from layer no.:"
		variable lastlayer=nr-1
		prompt lastlayer, "Finish at layer no.:"
		variable method=2
		prompt method, "method",popup "constant drift;linear drift;sqrt drift"
		DoPrompt "cursor AB drift correction  (moves B to A):", firstlayer,lastlayer,method
		if (V_flag==0)
			if (method==3)
				String yStr
				Prompt yStr, "enter wave",popup WaveList("*S*V*",";","")
				doPrompt "left axis input wave" yStr
				wave yw=$yStr
			endif
			variable ilayer
			ilayer=firstlayer
			do
				if (method==1)
					pw[ilayer]=(p0-p1)
					qw[ilayer]=(q0-q1)		
				endif
				if (method==2)
					pw[ilayer]=(p1-p0)*ilayer/(lastlayer-firstlayer)
					qw[ilayer]=(q1-q0)*ilayer/(lastlayer-firstlayer)
				endif
				if (method==3)
					pw[ilayer]=(p1-p0)*sqrt(yw[ilayer]-yw[firstlayer])/sqrt(yw[lastlayer]-yw[firstlayer])
					qw[ilayer]=(q1-q0)*sqrt(yw[ilayer]-yw[firstlayer])/sqrt(yw[lastlayer]-yw[firstlayer])
				endif
				ilayer+=1
			while (ilayer<=lastlayer)
			// display drift waves in dedicated table
			SIS_tableToFront(tableStr,pStr)
			SIS_tableToFront(tableStr,qStr)
		endif 
	else	
		print "put cursor A and B on initial and final point of the drift respectively ."
	endif
end

Function DCR_csrAB_layer()
	// data folder and stack
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	String stackStr=SIS_getImageStackName()
	wave wstack=$stackStr
	variable nr=dimSize($stackStr,2)
	NVAR layer=root:$(dfStr):layer
	// pause for cursors
	String winStr=SIS_getWinName()
	ShowInfo
	SIS_PauseForCursors(winStr,"csrA (B) reference (same point)")
	// create drift waves 
	String pStr="DC"+stackStr+"_P"
	String qStr="DC"+stackStr+"_Q"
	if ((exists(pStr)==0)&&(exists(qStr)==0))
		make/n=(nr)/o $pStr,$qStr 
	else
		redimension/n=(nr) $pStr,$qStr
	endif
	wave pw=$pStr
	wave qw=$qStr
	String tableStr="DC"+stackStr
	// check that cursor A and B are on the image
	if ((stringmatch(CsrInfo(A),"")==0) && (stringmatch(CsrInfo(B),"")==0))
		// cursor positions
		variable q0=qcsr(A)
		variable q1=qcsr(B)
		variable p0=pcsr(A)
		variable p1=pcsr(B)
		pw[layer]=-(p0-p1)
		qw[layer]=-(q0-q1)		
		// display drift waves in dedicated table
		SIS_tableToFront(tableStr,pStr)
		SIS_tableToFront(tableStr,qStr)
	endif
end

Function DCR_MarqueeDriftCorrection(): GraphMarquee 
	String ctrlname
	// data folder and stack
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	String stackStr=SIS_getImageStackName()
	wave wstack=$stackStr
	NVAR layer=root:$(dfStr):layer
	// marquee
	GetMarquee/Z/K left, top
	if (V_Flag == 0)
		Print "There is no marquee"
	else
		NVAR dcrSmoothFlag = root:SIS:dcrSmoothFlag	
		NVAR dcrSmoothFactor = root:SIS:dcrSmoothFactor
		NVAR dcrAbsFlag = root:SIS:dcrAbsFlag
		NVAR dcrFilterFlag = root:SIS:dcrFilterFlag
		NVAR dcrMethodFlag = root:SIS:dcrMethodFlag
		NVAR dcrFitwidth = root:SIS:dcrFitwidth
		SetDataFolder root:SIS
		if (!NVAR_Exists(dcrSmoothFlag))	  // No such global numeric variable?
			variable/G dcrSmoothFlag = 2    // Create and initialize it
		endif
		if (!NVAR_Exists(dcrSmoothFactor))     // No such global numeric variable?
			variable/G dcrSmoothFactor = 5    // Create and initialize it
		endif
		if (!NVAR_Exists(dcrAbsFlag))	     // No such global numeric variable?
			variable/G dcrAbsFlag = 1      // Create and initialize it
		endif
		if (!NVAR_Exists(dcrFilterFlag))	  // No such global numeric variable?
			variable/G dcrFilterFlag = 2    // Create and initialize it
		endif
		if (!NVAR_Exists(dcrMethodFlag))     // No such global numeric variable?
			variable/G dcrMethodFlag = 2    // Create and initialize it
		endif
		if (!NVAR_Exists(dcrFitwidth))	     // No such global numeric variable?
			variable/G dcrFitwidth = 3      // Create and initialize it
		endif		
		SetDataFolder root:$(dfStr)
		variable nr=dimSize($stackStr,2)
		// input
		variable firstlayer=layer
		prompt firstlayer, "Start from layer no.:"
		variable lastlayer=nr-1
		prompt lastlayer, "Finish at layer no.:"
		variable smoothFlag=dcrSmoothFlag
		prompt smoothFlag, "smoothing of cross-correlation function?",popup "no;yes;"
		variable smoothFactor=dcrSmoothFactor
		prompt smoothFactor, "smooth factor (must be odd)"
		variable absFlag=dcrAbsFlag
		prompt absFlag, "abs of cross-correlation function?",popup "no;yes;"
		variable filterFlag=dcrFilterFlag
		prompt filterFlag, "perform image filtering when calculating image correlation?",popup "none;find edges;sharpen"
		variable methodFlag=dcrMethodFlag
		prompt methodFlag, "fit of cross-correlation function?",popup "no;yes;"	
		variable fitwidth=dcrFitwidth
		prompt fitwidth, "width interval for fitting cross correlation peak?",popup "1/2;1/4;1/6;1/8;"	
		DoPrompt "drift correction input", firstlayer, lastlayer, smoothFlag, smoothFactor, absFlag, filterFlag, methodFlag, fitwidth
		if (V_flag==0)
			// storing globals
			dcrSmoothFlag=smoothFlag
			dcrSmoothFactor=smoothFactor
			dcrAbsFlag=absFlag
			dcrFilterFlag=filterFlag
			dcrMethodFlag=methodFlag
			dcrFitwidth=fitwidth
			// 
			firstlayer=max(0,firstlayer)
			lastlayer=min(lastlayer,nr-1)
			fitwidth=fitwidth*2
			// marquee position
			variable q0=2*round(V_top/2)
			variable q1=2*round(V_bottom/2)
			variable p0=2*round(V_left/2)
			variable p1=2*round(V_right/2)
			// ensures rows are an even number
			//  this is needed in order to compute FFT
			// create drift waves 
			String pStr="DC"+stackStr+"_P"
			String qStr="DC"+stackStr+"_Q"
			if ((exists(pStr)==0)&&(exists(qStr)==0))
				make/n=(nr)/o $pStr,$qStr
			else
				redimension/n=(nr) $pStr,$qStr
			endif
			wave pw=$pStr
			wave qw=$qStr
			String tableStr="DC"+stackStr
			// get reference image
			ImageTransform/P=(layer)/PTYP=0 getPlane $stackStr
			wave M_ImagePlane 	
			SIS_redimensionImage("M_ImagePlane",q0,q1,p0,p1) 
			duplicate/o M_imagePlane wref
			// computes and edit p,q waves of drifts
			variable ilayer=firstlayer
			variable/C pqdrift
			do
				ImageTransform/P=(ilayer)/PTYP=0 getPlane $stackStr	
				SIS_redimensionImage("M_ImagePlane",q0,q1,p0,p1) 
				duplicate/o M_imagePlane wtmp
				pqdrift=DCR_2Dcorrelation("wref","wtmp", smoothFlag, smoothFactor, absFlag, filterFlag, methodFlag, fitwidth)
				pw[ilayer]=real(pqdrift)
				qw[ilayer]=imag(pqdrift)
				ilayer+=1
			while (ilayer<=lastlayer)
			// display drift waves in dedicated table
			SIS_tableToFront(tableStr,pStr)
			SIS_tableToFront(tableStr,qStr)
			killwaves/z wref,wtmp,M_ImagePlane
		endif	
	endif 
end

Function/C DCR_2Dcorrelation(imageStr1, imageStr2, smoothFlag, smoothFactor, absFlag, filterFlag, methodFlag, fitwidth)
	String imageStr1,imageStr2
	variable smoothFlag, smoothFactor, absFlag, filterFlag, methodFlag, fitwidth
	wave w0=$imageStr1
	duplicate/o w0,w1
	wave w2=$imageStr2
	variable np=dimSize(w1,0)
	variable nq=dimSize(w1,1)
	// print np,nq
	wavestats/Q w1
	variable V_avg1=V_avg
	wavestats/Q w2
	variable V_avg2=V_avg
	w1-=V_avg1
	w2-=V_avg2
	// apply filter	
	if (filterFlag == 2) 
		imageFilter findEdges w1 //ImageFilter sharpen w0
		ImageFilter findEdges w2 //ImageFilter sharpen w1
	endif
	if (filterFlag == 3) 
		ImageFilter sharpen w1
		ImageFilter sharpen w2
	endif
	duplicate/o/C w1 fft1
	duplicate/o/C w2 fft2
	// FFT
	make/n=(numpnts(fft1)/2)/o invfft
	FFT/OUT=1/DEST=fft1/WINF=Hanning w1
	FFT/OUT=1/DEST=fft2/WINF=Hanning w2
	// performs 2D correlation
      	fft1=conj(fft1)
      	fft1*=fft2
	IFFT/DEST=invfft/R fft1
	// swap wave to put cross-correlation peak in the centre
	ImageTransform Swap invfft
	if (absFlag==2)
		invfft=abs(invfft)
	endif
	// smooth to avoid beeing fooled by noise	
	if (smoothFlag==2)
		Smooth/S=2 (smoothFactor), invfft
	endif
	//find local maximum
	ImageStats invfft
	// computes drifts
	variable pdrift,qdrift
	if (methodFlag==1)
	// quick method
       	qdrift = V_MaxColLoc -nq/2 //-nq
       	pdrift = V_MaxRowLoc-np/2  //-np
       endif
       // slow but more precise
       if (methodFlag==2)
		variable np1=max(0,V_MaxRowLoc-np/fitwidth)
		variable np2=min(V_MaxRowLoc+np/fitwidth,np-1)
		variable nq1=max(0,V_MaxColLoc-nq/fitwidth)
		variable nq2=min(V_MaxColLoc+nq/fitwidth,nq-1)
		CurveFit/NTHR=0/Q Gauss2D  invfft[np1,np2][nq1,nq2] /D 
		wave W_coef
		pdrift=W_coef[2]-np/2
		qdrift=W_coef[4]-nq/2
	endif
	killwaves/z w1,w2,fft1,fft2,invfft,fit_invfft
	return cmplx(pdrift,qdrift)
end

Function DCR_preview(): GraphMarquee 
	String ctrlname
	// data folder and stack
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	String stackStr=SIS_getImageStackName()
	wave wstack=$stackStr
	NVAR layer=root:$(dfStr):layer	
	variable nr=dimSize($stackStr,2)
	// get drift waves 
	String pStr="DC"+stackStr+"_P"
	String qStr="DC"+stackStr+"_Q"
	wave pw=$pStr
	wave qw=$qStr
	// display using DC drifts p,q
	variable ilayer=0
	ImageTransform/P=(ilayer)/PTYP=0 getPlane $stackStr
	wave M_imagePlane
	variable dx,dy
	do
		dx=-pw[ilayer]
		dy=-qw[ilayer]
		ImageTransform/P=(ilayer)/PTYP=0 getPlane $stackStr
		ImageInterpolate/TRNS={scaleShift,dx,1,dy,1} Resample M_imagePlane
		if (ilayer==0)
			SIS_imageToFront("DriftCorrectionPreview","M_InterpolatedImage")
		endif
		doUpdate
		ilayer+=1
	while (ilayer < nr)
	Dowindow/K DriftCorrectionPreview
end

Function DCR_doDriftCorrection(): GraphMarquee 
	String ctrlname
	// data folder and stack
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	String stackStr=SIS_getImageStackName()
	wave wstack=$stackStr
	NVAR layer=root:$(dfStr):layer	
	variable nr=dimSize($stackStr,2)
	// get drift waves 
	String pStr="DC"+stackStr+"_P"
	String qStr="DC"+stackStr+"_Q"
	wave pw=$pStr
	wave qw=$qStr
	variable ilayer=0
	variable dx,dy
	do
		dx=-pw[ilayer]
		dy=-qw[ilayer]
		if ((dx!=0)||(dy!=0))
			ImageTransform/P=(ilayer)/PTYP=0 getPlane $stackStr
			ImageInterpolate/TRNS={scaleShift,dx,1,dy,1} Resample M_imagePlane
			ImageTransform/P=(ilayer)/PTYP=0/D=M_InterpolatedImage setPlane $stackStr
		endif
		ilayer+=1
	while (ilayer < nr)
	killwaves/z M_InterpolatedImage, M_imagePlane
	pw=0
	qw=0
	String tableStr="DC"+stackStr
	DoWindow/K $tableStr
end

Function DCR_driftlayer(dx,dy)
	variable dx,dy
	// data folder and stack
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	String stackStr=SIS_getImageStackName()
	wave wstack=$stackStr
	// get layer
	NVAR layer=root:$(dfStr):layer
	ImageTransform/P=(layer)/PTYP=0 getPlane $stackStr
	// resample
	ImageInterpolate/TRNS={scaleShift,dx,1,dy,1} Resample M_imagePlane
	// put the layer back
	ImageTransform/P=(layer)/PTYP=0/D=M_InterpolatedImage setPlane $stackStr 
end

Function DCR_coadd_Layers()
	// data folder and stack
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	String stackStr=SIS_getImageStackName()
	wave wstack=$stackStr
	NVAR layer=root:$(dfStr):layer
	variable np=dimSize($stackStr,0)
	variable nq=dimSize($stackStr,1)
	variable nr=dimSize($stackStr,2)
	//  input
	variable method=1
	prompt method, "how many layers",popup "all layers;first half;second half;select;"
	doPrompt "coadd input", method
	if (V_flag==0)
		String graphStr="AV"+dfStr
		String aveStr=dfStr+"_AVG"
		make/n=(np,nq)/o $aveStr
		wave wav=$aveStr
		if (method==1)
			ImageTransform averageImage $stackStr
			duplicate/o M_AveImage $aveStr
			killwaves/z M_AveImage,M_StdvImage
			// display
			SIS_imageToFront(graphStr,aveStr) 		
			SIS_ImageControls()
		endif
		if (method==2)
			DCR_coadd(wstack,wav,0,round(nr/2)-1)
			// display
			SIS_imageToFront(graphStr,aveStr) 	
			SIS_ImageControls()
		endif
		if (method==3)
			DCR_coadd(wstack,wav,round(nr/2),nr-1)
			// display
			SIS_imageToFront(graphStr,aveStr) 	
			SIS_ImageControls()
		endif
		if (method==4)
			variable firstlayer=0
			prompt firstlayer, "Start from layer no.:"
			variable lastlayer=nr-1
			prompt lastlayer, "Finish at layer no.:"
			DoPrompt "Coadd inupt: ", firstlayer, lastlayer
			if (V_flag==0)
				DCR_coadd(wstack,wav,firstlayer,lastlayer)
				// display
				SIS_imageToFront(graphStr,aveStr) 	
				SIS_ImageControls()
			endif
		endif
	endif
End

Function DCR_coAddAroundALayer()
	// data folder and stack
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr)
	String stackStr=SIS_getImageStackName()
	wave wstack=$stackStr
	NVAR layer=root:$(dfStr):layer
	variable np=dimSize($stackStr,0)
	variable nq=dimSize($stackStr,1)
	variable nr=dimSize($stackStr,2)
	variable selectedlayer=layer
	prompt selectedlayer, "Selected layer:"
	variable width=2
	prompt width, "width:"
	DoPrompt "Coadd inupt: ", selectedlayer, width
	if (V_flag==0)
		String graphStr="AV"+num2Str(selectedlayer)+"_"+dfStr
		String aveStr=dfStr+"_AVG"+num2Str(selectedlayer)
		make/n=(np,nq)/o $aveStr=0
		wave wav=$aveStr
		variable firstlayer=selectedlayer-width
		if (firstlayer<0)
			firstlayer=0
		endif
		variable lastlayer=selectedlayer+width
		if (lastlayer>=nr)
			lastlayer=nr-1
		endif
		DCR_coadd(wstack,wav,firstlayer,lastlayer)
		// display
		SIS_imageToFront(graphStr,aveStr) 	
		SIS_ImageControls()
	endif
End

Function DCR_coadd(wstack,wavg,istart,iend)
	wave wstack,wavg
	variable istart,iend
	variable ilayer=istart
	do
		ImageTransform/P=(ilayer)/PTYP=0 getPlane wstack
		wave M_imagePlane
		wavg+=M_imagePlane	
		ilayer+=1
	while (ilayer<=iend)
	wavg/=((iend-istart)+1)
	killwaves/z M_imagePlane
End